/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "defaultassociator.h"


namespace GerQ {

  DefaultAssociator::DefaultAssociator(
    AssoAdapter *aAdapter, DataContainer *aCtrYin, DataContainer *aCtrYan,
    void (*aAssociate)(DataObject*, DataObject*),
    void (*aDissociate)(DataObject*, DataObject*),
    void (*aGetYanData)(DataObject*, AssoData&),
    void (*aGetYinData)(DataObject*, AssoData&)
  ): Associator(aAdapter, aCtrYin, aCtrYan) {
    fAssociate = aAssociate;
    fDissociate = aDissociate;
    fGetYinData = aGetYinData;
    fGetYanData = aGetYanData;
  } // DefaultAssociator::DefaultAssociator

  void DefaultAssociator::associate(DataObject *aYin, DataObject *aYan) {
    (*fAssociate)(aYin, aYan);
  } // DefaultAssociator::associate

  void DefaultAssociator::dissociate(DataObject *aYin, DataObject *aYan) {
    (*fDissociate)(aYin, aYan);
  } // DefaultAssociator::dissociate

  void DefaultAssociator::getYanData(DataObject *aYin, AssoData &aYanData) {
    (*fGetYanData)(aYin, aYanData);
  } // DefaultAssociator::getYanData

  void DefaultAssociator::getYinData(DataObject *aYan, AssoData &aYinData) {
    (*fGetYinData)(aYan, aYinData);
  } // DefaultAssociator::getYinData

} // GerQ
