/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "defaultinstcontainer.h"

#include "sqldataadapter.h"


namespace GerQ {

  DefaultInstContainer::DefaultInstContainer(
    DataAdapter *aAdapter, DataObject* (*aCreateData)()
  ): InstContainer(aAdapter) {
    fCreateData = aCreateData;
  } // DefaultInstContainer::DefaultInstContainer

  DefaultInstContainer::~DefaultInstContainer() {
    /*
      Default containers are registered with the global container table with
      the database table name as their key. So get the table name from the
      adapter:
    */
    SqlDataAdapter *adapter = (SqlDataAdapter*) getAdapter();
    DataContainer::removeDefaultContainer(adapter -> getTableName());
    /*
      Since in the functions to create default containers, the adapter is only
      created as "owned" by the container, the container is also responsible to
      deallocate it again:
    */
    delete adapter;
  } // DefaultInstContainer::~DefaultInstContainer

  DataObject* DefaultInstContainer::createData() {
    return (*fCreateData)();
  } // DefaultInstContainer::createData

} // GerQ
