/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  GerQUtils is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "gerqutilsbase.h"

#include <stdexcept>


namespace GerQ {

  QString doubleToStr(
    double aValue, const char* aFormat, double aMin, double aMax, int aDigitsEx
  ) {
    QString format;
    if (aValue < aMin || aValue >= aMax) {
      format = QString().sprintf("%%0.%ie", aDigitsEx);
    } else {
      format = aFormat;
    } // if
    return QString().sprintf(qPrintable(format), aValue);
  } // doubleToStr

  void initEditDouble(
    QLineEdit *aEdit, double aValue, const char* aFormat,
    double aMin, double aMax, int aDigitsEx
  ) {
/*!!!
    QString format;
    if (aValue < aMin || aValue >= aMax) {
      format = QString().sprintf("%%0.%ie", aDigitsEx);
    } else {
      format = aFormat;
    } // if
    QString text = QString().sprintf(qPrintable(format), aValue);
    initEditText(aEdit, text);
!!!*/
    initEditText(aEdit, doubleToStr(aValue, aFormat, aMin, aMax, aDigitsEx));
  } // initEditDouble

  void initEditDouble(
    QLineEdit *aEdit, double aValue, bool aInvalid, const char* aFormat,
    double aMin, double aMax, int aDigitsEx
  ) {
    if (aInvalid) {
      initEditText(aEdit, "");
      return;
    } // if
/*!!!
    QString format;
    if (aValue < aMin || aValue >= aMax) {
      format = QString().sprintf("%%0.%ie", aDigitsEx);
    } else {
      format = aFormat;
    } // if
    QString text = QString().sprintf(qPrintable(format), aValue);
    initEditText(aEdit, text);
!!!*/
    initEditText(aEdit, doubleToStr(aValue, aFormat, aMin, aMax, aDigitsEx));
  } // initEditDouble

  void initEditText(
    QLineEdit *aEdit, const QString &aText, bool aReadOnly, bool aEnabled
  ) {
    if (!aEdit) {
      return;
    } // if
    aEdit -> setText(aText);
    aEdit -> setReadOnly(aReadOnly);
    aEdit -> setEnabled(aEnabled);
  } // initEditText

  double readDouble(QLineEdit *aEdit, double aInvalid) {
    try {
      return readDouble(aEdit -> text(), aInvalid);
    } catch (std::invalid_argument &aException) {
      throw invalid_input(aException.what(), aEdit);
    } // try
  } // readDouble

  double readDouble(const QString &aValue, double aInvalid) {
    if (aValue == "") {
      return aInvalid;
    } // if
    bool ok = true;
    double x = aValue.toDouble(&ok);
    if (ok) {
      return x;
    } else {
      throw std::invalid_argument(qPrintable(QString().sprintf(
        "Not a valid double: \"%s\"", qPrintable(aValue))));
    } // if
  } // readDouble

  int readInt(QLineEdit *aEdit) {
    try {
      return readInt(aEdit -> text());
    } catch (std::invalid_argument &aException) {
      throw invalid_input(aException.what(), aEdit);
    } // try
  } // readInt

  int readInt(const QString &aValue) {
    if (aValue == "") {
      return 0;
    } // if
    bool ok = true;
    int i = aValue.toInt(&ok);
    if (ok) {
      return i;
    } else {
      throw std::invalid_argument(qPrintable(QString().sprintf(
        "Not a valid int: \"%s\"", qPrintable(aValue))));
    } // if
  } // readInt

  int viewGetRow(QAbstractItemView *aView) {
    return aView -> currentIndex().row();
  } // viewGetRow

  void viewSetRow(QAbstractItemView *aView, int aRow) {
    int col = aView -> currentIndex().column();
    if (aRow < 0) {
      aView -> setCurrentIndex(QModelIndex());
    } else {
      if (col < 0) {
        col = 0;
      } // if
      QModelIndex idx = aView -> model() -> index(aRow, col);
      aView -> setCurrentIndex(idx);
    } // if
  } // viewSetRow

} // GerQ
