/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERQ_GERQUTILSBASE_H
#define GERQ_GERQUTILSBASE_H


#include <exception>
#include <iostream>
#include <QAbstractItemView>
#include <QLineEdit>
#include <QListView>
#include <QString>
#include <QTableView>
#include <stdexcept>


#define __NO_ERROR__ "GERQ: NO ERROR"


namespace GerQ {

  class DataObject;
  class DataContainer;
    class InstContainer;
      class DefaultInstContainer;
    class RefContainer;
      class DefaultRefContainer;
  class DataAdapter;
    class SqlDataAdapter;
      class DefaultSqlDataAdapter;
  class Associator;
    class DefaultAssociator;
  class AssoAdapter;
    class DefaultSqlAssoAdapter;

  class no_error: public std::exception {
    public:
      virtual const char* what() const throw() { return __NO_ERROR__; }
  }; // no_error

  class invalid_input: public std::invalid_argument {
    public:
      invalid_input(const char* aWhat, QLineEdit *aEdit):
        std::invalid_argument(aWhat) { fEdit = aEdit; }
      QLineEdit *getEdit() { return fEdit; }
    private:
      QLineEdit *fEdit;
  }; // invalid_input

  extern int readInt(const QString &aValue);
  extern double readDouble(const QString &aValue, double aInvalid = 0.0);
  extern int readInt(QLineEdit *aEdit);
  extern double readDouble(QLineEdit *aEdit, double aInvalid = 0.0);

  extern QString doubleToStr(double aValue, const char* aFormat,
    double aMin = 0.0, double aMax = 1.0e+32, int aDigitsEx = 2);

  extern void initEditText(QLineEdit *aEdit, const QString &aText,
    bool aReadOnly = false, bool aEnabled = true);
  extern void initEditDouble(QLineEdit *aEdit, double aValue,
    const char* aFormat,
    double aMin = 0.0, double aMax = 1.0e+32, int aDigitsEx = 2);
  extern void initEditDouble(QLineEdit *aEdit, double aValue, bool aInvalid,
    const char* aFormat,
    double aMin = 0.0, double aMax = 1.0e+32, int aDigitsEx = 2);

  extern int viewGetRow(QAbstractItemView *aView);
  extern void viewSetRow(QAbstractItemView *aView, int aRow);

} // GerQ


#define ABSTRACT 0;

#define FINALLY throw GerQ::no_error(); } catch (std::exception &exc) { if(strcmp(exc.what(), __NO_ERROR__) != 0) throw exc;
#define FINISH throw GerQ::no_error()


#endif
