/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "instcontainer.h"

#include "dataobject.h"
#include "refcontainer.h"


namespace GerQ {

  InstContainer::InstContainer(DataAdapter *aAdapter):
  DataContainer(aAdapter) {}

  InstContainer::~InstContainer() {
    while (getCount() > 0) {
      DataObject *data = *(beginData());
      delete data;
    } // while
  } // InstContainer::~InstContainer

  DataObject* InstContainer::newData(DataBuffer *aBuffer) {
    QVariant aID;
    /* Let the adapter create an ID and "make room" for the new data: */
    getAdapter() -> newData(aID);
    /* Now, create and store the data object itself: */
    DataObject *result = createData();
    result -> fID = aID;
    result -> fContainer = this;
    fData.insert(aID.toString(), result);
    RefContainer *parent = getParent();
    while (parent) {
      parent -> putNewData(result);
      parent = parent -> getParent();
    } // while
    /*
      If data was passed to fill the newly created data object with, do so now:
    */
    if (aBuffer) {
      aBuffer -> fill(result);
    } // if
    /*
      Whether filled from outside or just left as the constructor created it,
      give the data object to the adapter now for persistent storage:
    */
    result -> write();
    return result;
  } // InstContainer::newData

  DataObject* InstContainer::yetUnknownData(QVariant aID) {
    if (aID == getNullID()) {
      return NULL;
    } // if
    DataObject *result = createData();
    result -> fID = aID;
    result -> fContainer = this;
    /*
      Only return a result, if the adapter was able to read valid data for the
      ID requested:
    */
    if (result -> read()) {
      return result;
    } else {
      delete result;
      return NULL;
    } // if
  } // InstContainer::yetUnknownData

} // GerQ
